/**************************************************************************/
/*                                Flow4J                                  */
/*             A visual process flow configuration framework              */
/*                         (c) Alex Greif 2003                            */
/*                                                                        */
/*  This program is free software; you can redistribute it and/or modify  */
/*  it under the terms of the GNU General Public License as published by  */
/*  the Free Software Foundation; either version 2 of the License, or     */
/*  (at your option) any later version.                                   */
/*                                                                        */
/**************************************************************************/

package flow4j.flow;

import java.util.*;

import bsh.*;



/**	AbstractFlow is the superclass of all flows.
 It hold a BeanShell Interpreter for possible decision evaluations.
 */
public abstract class AbstractFlow implements IFlow {

	/**	The BeanShell interpreter that stores all scripted methods needed
	 for the evaluation of decisions in the flow.
	 */
	private Interpreter _interpreter;


	/**	Returns the interpreter member of this flow.
	 The instance will be created lazily.
	 */
	private Interpreter getInterpreter() {
		if (_interpreter == null)
			_interpreter = new Interpreter();

		return _interpreter;
	}



	/**	Creates dynamically a BeanShell scripted method with the
	 appropriate method name, argument list and decision return statement
	 */
	protected final void createDecisionMethod(String decisionMethodName,
											  String stmnt,
											  List decisionArgs)
	{
		StringBuffer buf = new StringBuffer();

		buf.append(	"boolean ").append(decisionMethodName).append("(");
		for (Iterator args = decisionArgs.iterator(); args.hasNext();)
			buf.append("Object ").append(args.next())
				.append(args.hasNext() ? "," : "");

		buf.append(") {return ");
		buf.append(stmnt);
		buf.append(";}");

		try {
			getInterpreter().eval(buf.toString());
		} catch(EvalError e) {
			e.printStackTrace();
		}
	}



	/**	Calls the BeanShell scripted method to evaluate the statement.
	 The argument list is dynamically constructed. Keys are in the decisionmethodArgs
	 and the values a re taken from the dictionary
	 */
	final protected boolean evalBooleanStmt(String decisionMethodName,
											List decisionmethodArgs,
											FlowDictionary dictionary)
	{
		Object[] argsArray = new Object [decisionmethodArgs.size()];

		for (int i = 0; i < decisionmethodArgs.size(); i++)
			argsArray[i] = dictionary.get(decisionmethodArgs.get(i));

		try {
			This tthis = (This)getInterpreter().get("this");
			Object o = tthis.invokeMethod(decisionMethodName, argsArray);
			return new Boolean("" + o).booleanValue();

		} catch (Exception e) {
			e.printStackTrace();
		}

		return true;
	}






}

