/**************************************************************************/
/*                                Flow4J                                  */
/*             A visual process flow configuration framework              */
/*                         (c) Alex Greif 2003                            */
/*                                                                        */
/*  This program is free software; you can redistribute it and/or modify  */
/*  it under the terms of the GNU General Public License as published by  */
/*  the Free Software Foundation; either version 2 of the License, or     */
/*  (at your option) any later version.                                   */
/*                                                                        */
/**************************************************************************/

package flow4j.flow;

import java.util.*;
import java.io.*;




/**
 */


public class FlowException extends RuntimeException {

	/** A wrapped <code>Throwable</code> */
	protected Throwable _cause;

	/**
	 * <p>
	 * This will create an <code>Exception</code>.
	 * </p>
	 */
	public FlowException() {
		super("Error occurred in the Flow4J framework.");
	}

	/**
	 * <p>
	 * This will create an <code>Exception</code> with the given message.
	 * </p>
	 *
	 * @param message <code>String</code> message indicating
	 *                the problem that occurred.
	 */
	public FlowException(String message)  {
		super(message);
	}

	/**
	 * <p>
	 * This will create an <code>Exception</code> with the given message
	 *   and wrap another <code>Exception</code>.  This is useful when
	 *   the originating <code>Exception</code> should be held on to.
	 * </p>
	 *
	 * @param message <code>String</code> message indicating
	 *                the problem that occurred.
	 * @param exception <code>Exception</code> that caused this
	 *                  to be thrown.
	 */
	public FlowException(String message, Throwable cause)  {
		super(message);
		_cause = cause;
	}

	/**
	 * <p>
	 * This returns the message for the <code>Exception</code>. If
	 *   there is a root cause, the message associated with the root
	 *   <code>Exception</code> is appended.
	 * </p>
	 *
	 * @return <code>String</code> - message for <code>Exception</code>.
	 */
	public String getMessage() {
		if (_cause != null) {
			return super.getMessage() + ": " + _cause.getMessage();
		} else {
			return super.getMessage();
		}
	}

	/**
	 * <p>
	 * This prints the stack trace of the <code>Exception</code>. If
	 *   there is a root cause, the stack trace of the root
	 *   <code>Exception</code> is printed right after.
	 * </p>
	 */
	public void printStackTrace() {
		super.printStackTrace();
		if (_cause != null) {
			System.err.print("Root cause: ");
			_cause.printStackTrace();
		}
	}

	/**
	 * <p>
	 * This prints the stack trace of the <code>Exception</code> to the given
	 *   PrintStream. If there is a root cause, the stack trace of the root
	 *   <code>Exception</code> is printed right after.
	 * </p>
	 */
	public void printStackTrace(PrintStream s) {
		super.printStackTrace(s);
		if (_cause != null) {
			s.print("Root cause: ");
			_cause.printStackTrace(s);
		}
	}

	/**
	 * <p>
	 * This prints the stack trace of the <code>Exception</code> to the given
	 *   PrintWriter. If there is a root cause, the stack trace of the root
	 *   <code>Exception</code> is printed right after.
	 * </p>
	 */
	public void printStackTrace(PrintWriter w) {
		super.printStackTrace(w);
		if (_cause != null) {
			w.print("Root cause: ");
			_cause.printStackTrace(w);
		}
	}

	/**
	 * <p>
	 * This will return the root cause <code>Throwable</code>, or null
	 *   if one does not exist.
	 * </p>
	 *
	 * @@return <code>Throwable</code> - the wrapped <code>Throwable</code>.
	 */
	public Throwable getCause()  {
		return _cause;
	}



}

